from time import sleep
#from GH.dht11 import DHT11 #Current config is using two analog sensors

class GHTemperature(object):
    """A class to read IoT Greenhouse temperature.
    """
    
    _analogTempSensor_in = None
    _analogTempSensor_out = None

    def __init__(self, analog_temp_channel_in, analog_temp_channel_out):
        """Initializes an IoT Greenhouse temperature service.
        
        Wrapper class for analog temperature channels
        :param analog_temp_channel_in: reference to inside temperature analog channel
        :param analog_temp_channel_out: reference to outside temperature analog channel
        :returns: GHTemperature object
        """
        self._analogTempSensor_in = analog_temp_channel_in
        self._analogTempSensor_out = analog_temp_channel_out
        

    def convert_C_to_F(self, temp_C):
        """Converts Celsius to Fahrenheit.
        """
        return temp_C * 9/5 +32

    def convert_F_to_C(self, temp_F):
        """Converts Fahrenheit to Celsius.
        """
        return (temp_F - 32) * 5/9

    # Function to calculate temperature in C from LM35 data
    def _convert_to_temp(self, data):
        """Converts A/D data to temp C.
        """
        temp = ((data * 330)/float(1023))
        return temp

    def get_inside_temp_C(self):
        """Gets inside temperature in degrees C.
        """
        analog_value = self._analogTempSensor_in.get_value()
        temp_value_C = self._convert_to_temp(analog_value)
        return round(temp_value_C, 1)

    def get_inside_temp_F(self):
        """Gets inside temperature in degrees F.
        """
        temp_value_F = self.convert_C_to_F(self.get_inside_temp_C()) 
        return round(temp_value_F, 1)

    def get_outside_temp_C(self):
        """Gets outside temperature in degrees C.
        """
        analog_value = self._analogTempSensor_out.get_value()
        temp_value_C = self._convert_to_temp(analog_value)
        return round(temp_value_C, 1)

    def get_outside_temp_F(self):
        """Gets outside temperature in degrees F.
        """
        temp_value_F = self.convert_C_to_F(self.get_outside_temp_C()) 
        return round(temp_value_F, 1)



import datetime
from iot_gh.src.iot_gh.GHgpio import GHgpio
from iot_gh.src.iot_gh.GHAnalogService import GHAnalogService
from iot_gh.src.iot_gh.GHBuzzer import GHBuzzer
from iot_gh.src.iot_gh.GHFan import GHFan
from iot_gh.src.iot_gh.GHLamps import GHLamps
from iot_gh.src.iot_gh.GHServo import GHServo
from iot_gh.src.iot_gh.GHSwitches import GHSwitches
from iot_gh.src.iot_gh.GHTemperature import GHTemperature
from iot_gh.src.iot_gh.GHWebService import GHWebService
from iot_gh.src.iot_gh.IoTGreenhouse import IoTGreenhouse

class GHService(object):
    """A container class for IoT Greenhouse services.
    
    Provides references to component services.
    """
    
    # _pi = None      #pigpio connection
    # _spi = None
    # _url = None
    # _servo_ccw_limit = None
    # _servo_cw_limit = None
    
    # greenhouse = None
    # version = None
    
    # analog_service = None
    # ain_pot = None
    # ain_temp = None
    # ain_light = None
    # ain_aux = None
    # buzzer = None
    # fan = None
    # lamps = None
    # servo = None
    # switches = None
    # temperature = None
    # web_service = None

    def __init__(self, pi, spi):
        """Initializes an IoT Greenhouse container.
        """
        self._pi = pi
        self._spi = spi

        if not self._pi.connected:
            raise Exception("ERROR: unable to connect to pigpio")
        
        #self._read_config()
        self._make_components()
        # self.greenhouse = IoTGreenhouse()
        # self.update_greenhouse()

    # def _read_config(self):
    #     """Loads configuration file from user iot_gh directory.
    #     """
    #     try:
    #         config = configparser.ConfigParser()
            
    #         # if type(self._pi) is : #debug mode. Load test file
    #         #    config.read("./iot_gh_system.conf")
    #         # else:
    #         #     config.read(["iot_gh_system.conf", os.path.expanduser("~/.iot_gh/iot_gh_system.conf")],encoding="UTF8")
    #         self.version = config["IOT_GREENHOUSE"]["VERSION"]
    #         # self._url = config["IOT_GREENHOUSE"]["URL"]
    #         self._servo_ccw_limit = int(config["IOT_GREENHOUSE"]["SERVO_CCW_LIMIT"])
    #         self._servo_cw_limit = int(config["IOT_GREENHOUSE"]["SERVO_CW_LIMIT"])
    #     except Exception as ex:
    #         raise Exception("Unable to load IoT Greenhouse System Configuration. %s" % ex.args[0])
       
    def _make_components(self):
        """Makes component services for IoT Greenhouse container.
        """
        self.analog = GHAnalogService(self._pi, self._spi)
        self.buzzer = GHBuzzer(self._pi, GHgpio.BUZZER)
        self.fan = GHFan(self._pi, GHgpio.FAN)
        self.lamps = GHLamps(self._pi, GHgpio.RED_LED, GHgpio.WHITE_LED, GHgpio.RED_LED, GHgpio.DUAL_LED)
        self.servo = GHServo(self._pi, GHgpio.SERVO_PWM, self._servo_cw_limit, self._servo_ccw_limit)
        self.switches = GHSwitches(self._pi, GHgpio.SWITCH_PB, GHgpio.SWITCH_TOGGLE)
        self.temperature = GHTemperature(self.analog.aux, self.analog.temp)
        self.web_service = GHWebService(self)    
        
    # def _make_greenhouse(self, name=""):
    #     """Factory class to build IoT Greenhouse data object.
    #     """
    #     gh = IoTGreenhouse(name)
        
    #     #read greenhouse config values
    #     config = configparser.ConfigParser()
    #     if type(self._pi) is not pigpio.pi: #debug mode. Load test file
    #         config.read("./iot_gh.conf")
    #     else:
    #         config.read(["iot_gh.conf", os.path.expanduser("~/.iot_gh/iot_gh.conf")],encoding="UTF8")
    #     gh.house_id = config["IOT_GREENHOUSE"]["HOUSE_ID"]
    #     gh.group_id = config["IOT_GREENHOUSE"]["GROUP_ID"]
    #     gh.house_number = config["IOT_GREENHOUSE"]["HOUSE_NUMBER"].zfill(2)
    #     gh.version = self.version

    #     return gh
    
    def update_greenhouse(self, greenhouse:IoTGreenhouse ):
        """Updates IoTGreenhouse object by reading all house 
        states and refreshing gh object.
        """
        try:
    
            greenhouse.led_red_state = self.lamps.red.get_state()
            greenhouse.led_red_status = self.lamps.red.get_status()
            greenhouse.led_white_state = self.lamps.white.get_state()
            greenhouse.led_white_status = self.lamps.white.get_status()
            greenhouse.led_dual_state =  self.lamps.dual.get_state()
            greenhouse.led_dual_status = self.lamps.dual.get_status()
            greenhouse.switch_pb_state = self.switches.push_button.get_state()
            greenhouse.switch_pb_status = self.switches.push_button.get_status()
            greenhouse.switch_toggle_state = self.switches.toggle.get_state()
            greenhouse.switch_toggle_status = self.switches.toggle.get_status()
            greenhouse.fan_state =  self.fan.get_state()
            greenhouse.fan_status =  self.fan.get_status()
            greenhouse.servo_position = self.servo.get_value()
            greenhouse.servo_status = self.servo.get_status()
            greenhouse.heater_state = self.lamps.white.get_state()
            greenhouse.heater_status = self.lamps.white.get_status()
            greenhouse.buzzer_state = self.buzzer.get_state()
            greenhouse.buzzer_status = self.buzzer.get_status()
            greenhouse.ain_pot_raw = self.analog.pot.get_value()
            greenhouse.ain_light_raw =  self.analog.light.get_value()
            greenhouse.ain_aux_raw =  self.analog.aux.get_value()

            greenhouse.temp_inside_C = self.temperature.get_inside_temp_C()
            greenhouse.temp_inside_F = self.temperature.get_inside_temp_F()
            greenhouse.temp_outside_C = self.temperature.get_outside_temp_C()
            greenhouse.temp_outside_F = self.temperature.get_outside_temp_F()
            
            greenhouse.last_update = datetime.datetime.now().replace(microsecond=0).isoformat(' ')
            
            self._make_message(greenhouse)
            

        except Exception as e:
            greenhouse.message = "Update exception: %s" % e.args[0]
            
    def _make_message(self, gh:IoTGreenhouse):
        m = ""
        if  gh.temp_outside_C > gh.temp_inside_C: 
            m = "Warning: The temperature outside the greenhouse is higher than the internal temperature."
        elif gh.servo_status == "CLOSED" and gh.fan_status == "ON":
            m = "Error: The fan is activated by louvers are closed."
        else:
            m = "Greenhouse status is normal."

        gh.message = m


import time
import os
from iot_gh.src.iot_gh.IoTGreenhouse import IoTGreenhouse
from airtable import Airtable

class GHWebService(object):
    """ Web connector for REST service. 
    Post data using json package.
    V2.1
    2/26/2022
    """
    POST_DELAY = 30     #Seconds between posts - throttle
    DATA_TABLE_NAME = 'GreenhouseData'
    DATA_TABLE_LOGS = 'GreenhouseLogs'
    
    def __init__(self, greenhouse:IoTGreenhouse):
        self.greenhouse = greenhouse
        api_key = os.environ['airtable_key']
        base_key = os.environ['airtable_iotgh_key']
        self._gh_data_table = Airtable(base_key, self.DATA_TABLE_NAME, api_key)
        self._gh_logs_table = Airtable(base_key, self.DATA_TABLE_LOGS, api_key)
        self.record_id = self.find_gh_record_id()
        self.last_post_time = 0
        
        # self.log_service = AppLogService(api_key, base_key)url, greenhouse_service):
        # self.gh_service = greenhouse_service
        # self.url = greenhouse_service._url
        
    def post_greenhouse(self):
        #throttle post to every 30 sec
        # gh = self.gh_service.greenhouse
        if time.time() > self.last_post_time + self.POST_DELAY:
            if not self.gh_is_valid():
                # log = AppLog(LOG_TYPE.ERROR, "Unable to add company name select. Company Name Select is None")
                # self.log_service.add_log(log)   
                pass
            else:
                try:
                    fields = self.get_fields()
                    if not self.record_id:
                        self._gh_data_table.insert(fields)
                    else:
                        self._gh_data_table.update({"id": self.record_id, "fields": fields})
                    self.last_post_time = time.time()
                except Exception as ex:
                    # log = AppLog(LOG_TYPE.ERROR, "Unable to add company. " + str(ex))
                    # self.log_service.add_log(log)
                    raise ex

    def gh_is_valid(self):
        return True

    def find_gh_record_id(self) -> str:
        "Return Company Name Select with specified name. Should be single record."
        records = self._gh_data_table.search('Name', self.greenhouse.name)
        if len(records) == 0:
            return ""
        elif len(records) == 1:
            return records["id"]
        else:
            # log = AppLog(LOG_TYPE.ERROR, "Duplicate Company Name records exist in Company Name Select.")
            # self.log_service.add_log(log)
            pass  

    def get_fields(self):
        fields = {"Name": self.greenhouse.name,
            "Temp In": self.greenhouse.temp_inside_F,
            "Temp Out": self.greenhouse.temp_outside_F,
            "Servo Status": self.greenhouse.servo_status,
            "Fan Status": self.greenhouse.fan_status,
            "Heater Status": self.greenhouse.led_white_status,  #white led simulates heater

            "Message": self.greenhouse.message}
        return fields

gh = IoTGreenhouse()
ws = GHWebService(gh)
ws.post_greenhouse()



from time import sleep

class GHBuzzer(object):
    """A class to access the IoT Greenhouse buzzer.
    """
    BUZZER_ON = False
    BUZZER_OFF = True
    BEEP_DELAY_ON = .5
    BEEP_DELAY_OFF = .3

    def __init__(self, pi, buzzer_gpio):
        """Initializes an IoT Greenhouse buzzer.
        
        :param pi: Reference to pigpio service.
        :param buzzer_gpio: GPIO port for buzzer. Use GHgpio definitions.
        :returns: GHBuzzer object
        """
        self._pi = pi
        self._gpio = buzzer_gpio

        self._pi.set_mode(self._gpio, self._pi.OUTPUT)        
        #buzzer off
        self._pi.write(self._gpio, GHBuzzer.BUZZER_OFF)
        
    def on(self):
        """Activate buzzer.
        """
        self._pi.write(self._gpio, GHBuzzer.BUZZER_ON)

    def off(self):
        """Deactivate buzzer.
        """
        self._pi.write(self._gpio, GHBuzzer.BUZZER_OFF)

    def beep(self, duration = BEEP_DELAY_ON):
        """Activates buzzer for specifed duration.
        
        :param duration float: Number of seconds to activate buzzer.
        """
        self.on()
        sleep(duration)
        self.off()
 
    def beeps(self, duration = BEEP_DELAY_ON, beep_count = 2):
        """Activates buzzer for multiple beeps.

        :param duration float: Number of seconds to sound buzzer.
        :param beep_count int: Number of times to beep.
        """
        for i in range(0, beep_count):
            if i > 0:
                sleep(GHBuzzer.BEEP_DELAY_OFF)
        self.beep(duration)
            
    def get_state(self):
        """Returns current state of buzzer.
        """
        return self._pi.read(self._gpio)

    def is_on(self):
        """Returns Boolean indicating buzzer is on.
        """
        return self.get_state() == GHBuzzer.BUZZER_ON

    def is_off(self):
        """Returns Boolean indicating buzzer is off
        """
        return self.get_state() == GHBuzzer.BUZZER_OFF

    def get_status(self):
        """Gets status of buzzer.

        :returns: Button status as string.
        """
        if self.get_state() == GHBuzzer.BUZZER_ON:
            return "ON"
        else:
            return "OFF"


from time import sleep

class GHSwitches(object):
    """A container class to access the IoT Greenhouse switches.
    """
    push_button = None
    toggle = None
    
    def __init__(self, pi, pb_switch_gpio, toggle_switch_gpio):
        """Initializes a container for IoT Greenhouse switches.
    
        Returns references to both push button and toggle switches

        :param pi: reference to pigpio service
        :param pb_switch_gpio: GPIO port for push button switch. Use GHgpio definitions.
        :param toggle_switch_gpio: GPIO port for toggle switch. Use GHgpio definitions.
        :returns: GHSwitches object
        """
        self.push_button = GHSwitch(pi, pb_switch_gpio)
        self.toggle = GHSwitch(pi, toggle_switch_gpio)

class GHSwitch(object):
    """A class to access IoT Greenhouse switch.
    """  
    SWITCH_ON = False
    SWITCH_OFF = True
       
    def __init__(self, pi, switch_gpio):
        """Initializes an IoT Greenhouse switch.
        
        Returns references to switch object. Used for both push button and toggle.

        :param pi: reference to pigpio service
        :param switch_gpio: GPIO port for switch. Use GHgpio definitions.
        :returns: GHSwitch object
        """
        self._pi = pi
        self._gpio = switch_gpio
        self._pi.set_mode(self._gpio, self._pi.INPUT)        
        
    def get_state(self, debounce=False):
        """Returns current state of switch. 
        """
        if not debounce:
            return self._pi.read(self._gpio)
        else:
            first_read = False
            second_read = True
            debouce_delay = .01
            while first_read != second_read:
                first_read = self._pi.read(self._gpio)
                sleep(debouce_delay)
                second_read = self._pi.read(self._gpio)
            return first_read

    def wait_for_press(self):
        """Waits for switch on or switch pressed state.
        """
        while self.get_state():
            sleep(.2)

    def wait_for_release(self):
        """Waits for switch off or switch released state.
        """
        while not self.get_state():
            sleep(.2)

    def wait_for_press_and_release(self):
        """Waits for switch press and released states.
        """
        self.wait_for_press()
        self.wait_for_release()

    def is_on(self):
        """Returns Boolean indicating switch is on.
        """
        return self.get_state() == GHSwitch.SWITCH_ON

    def is_off(self):
        """Returns Boolean indicating switch is off.
        """
        return self.get_state() == GHSwitch.SWITCH_OFF

    def get_status(self):
        """Gets status of switch.

        :returns: Switch status as string.
        """
        if self.get_state() == GHSwitch.SWITCH_ON:
            return "ON"
        else:
            return "OFF"




from iot_gh.GHService import GHService

class IoTGreenhouseService(GHService):
    """A container class for IoT Greenhouse services implemented on the 
    Raspberry Pi.
    """

    def __init__(self, pi=None):
        import spidev

        if not pi:
            import pigpio
            pi = pigpio.pi()
        spi = spidev.SpiDev()
        super().__init__(pi, spi)
